@@ -66,7 +66,7 @@ got_time_tick (GstPlay * play, gint64 time_nanos)
 static void
 got_stream_length (GstPlay * play, gint64 length_nanos)
 {
-  g_print ("got length %" G_GUINT64_FORMAT "\n", length_nanos);
+  g_print ("got length %llu\n", length_nanos);
   length = length_nanos;
 }
 
@@ -90,6 +90,13 @@ seek_timer (GstPlay * play)
   return FALSE;
 }
 
+static gboolean
+idle_iterate (GstPlay * play)
+{
+  gst_bin_iterate (GST_BIN (play));
+  return (GST_STATE (GST_ELEMENT (play)) == GST_STATE_PLAYING);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -161,6 +168,7 @@ main (int argc, char *argv[])
           GST_STATE_PLAYING) == GST_STATE_FAILURE)
     g_error ("Could not set state to PLAYING");
 
+  g_idle_add ((GSourceFunc) idle_iterate, play);
   g_timeout_add (20000, (GSourceFunc) seek_timer, play);
 
   g_main_loop_run (loop);
