@@ -170,7 +170,7 @@ gst_flacdec_class_init (FlacDecClass * klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gobject_class->dispose = gst_flacdec_dispose;
-  
+
   gstelement_class->change_state = gst_flacdec_change_state;
 }
 
@@ -467,26 +467,24 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
 
     flacdec->need_discont = FALSE;
 
-    if (!GST_PAD_CAPS (flacdec->srcpad)) {
-      if (flacdec->seek_pending) {
-        flacdec->total_samples = flacdec->seek_value;
-      }
+    if (flacdec->seek_pending) {
+      flacdec->total_samples = flacdec->seek_value;
+    }
 
-      if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
-        GST_DEBUG ("send discont");
+    if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
+      GST_DEBUG ("send discont");
 
-        format = GST_FORMAT_TIME;
-        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
-            flacdec->total_samples, &format, &time);
-        format = GST_FORMAT_BYTES;
-        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
-            flacdec->total_samples, &format, &bytes);
-        discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
-            GST_FORMAT_BYTES, bytes,
-            GST_FORMAT_DEFAULT, flacdec->total_samples, NULL);
+      format = GST_FORMAT_TIME;
+      gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+          flacdec->total_samples, &format, &time);
+      format = GST_FORMAT_BYTES;
+      gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+          flacdec->total_samples, &format, &bytes);
+      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
+          GST_FORMAT_BYTES, bytes,
+          GST_FORMAT_DEFAULT, flacdec->total_samples, NULL);
 
-        gst_pad_push (flacdec->srcpad, GST_DATA (discont));
-      }
+      gst_pad_push (flacdec->srcpad, GST_DATA (discont));
     }
   }
 
@@ -511,6 +509,8 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
     GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (outbuf));
     GST_BUFFER_TIMESTAMP (outbuf) =
         flacdec->total_samples * GST_SECOND / frame->header.sample_rate;
+    GST_BUFFER_DURATION (outbuf) =
+        samples * GST_SECOND / frame->header.sample_rate;
 
     if (depth == 8) {
       guint8 *outbuffer = (guint8 *) GST_BUFFER_DATA (outbuf);
