@@ -39,23 +39,6 @@ GST_DEBUG_CATEGORY (jpegenc_debug);
 
 #define JPEG_DEFAULT_QUALITY 85
 
-/* These macros are adapted from videotestsrc.c 
- *  and/or gst-plugins/gst/games/gstvideoimage.c */
-#define ROUND_UP_2(x)  (((x)+1)&~1)
-#define ROUND_UP_4(x)  (((x)+3)&~3)
-#define ROUND_UP_8(x)  (((x)+7)&~7)
-
-/* I420 */
-#define I420_Y_ROWSTRIDE(width) (ROUND_UP_4(width))
-#define I420_U_ROWSTRIDE(width) (ROUND_UP_8(width)/2)
-#define I420_V_ROWSTRIDE(width) ((ROUND_UP_8(I420_Y_ROWSTRIDE(width)))/2)
-
-#define I420_Y_OFFSET(w,h) (0)
-#define I420_U_OFFSET(w,h) (I420_Y_OFFSET(w,h)+(I420_Y_ROWSTRIDE(w)*ROUND_UP_2(h)))
-#define I420_V_OFFSET(w,h) (I420_U_OFFSET(w,h)+(I420_U_ROWSTRIDE(w)*ROUND_UP_2(h)/2))
-
-#define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*ROUND_UP_2(h)/2))
-
 /* JpegEnc signals and args */
 enum
 {
@@ -75,7 +58,6 @@ enum
 static void gst_jpegenc_base_init (gpointer g_class);
 static void gst_jpegenc_class_init (GstJpegEnc * klass);
 static void gst_jpegenc_init (GstJpegEnc * jpegenc);
-static void gst_jpegenc_finalize (GObject * object);
 
 static void gst_jpegenc_chain (GstPad * pad, GstData * _data);
 static GstPadLinkReturn gst_jpegenc_link (GstPad * pad, const GstCaps * caps);
@@ -86,8 +68,6 @@ static void gst_jpegenc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_jpegenc_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static GstElementStateReturn gst_jpegenc_change_state (GstElement * element);
-
 
 static GstElementClass *parent_class = NULL;
 static guint gst_jpegenc_signals[LAST_SIGNAL] = { 0 };
@@ -99,7 +79,7 @@ gst_jpegenc_get_type (void)
 
   if (!jpegenc_type) {
     static const GTypeInfo jpegenc_info = {
-      sizeof (GstJpegEncClass),
+      sizeof (GstJpegEnc),
       gst_jpegenc_base_init,
       NULL,
       (GClassInitFunc) gst_jpegenc_class_init,
@@ -173,9 +153,6 @@ gst_jpegenc_class_init (GstJpegEnc * klass)
 
   gobject_class->set_property = gst_jpegenc_set_property;
   gobject_class->get_property = gst_jpegenc_get_property;
-  gstelement_class->change_state = gst_jpegenc_change_state;
-
-  gobject_class->finalize = gst_jpegenc_finalize;
 
   GST_DEBUG_CATEGORY_INIT (jpegenc_debug, "jpegenc", 0,
       "JPEG encoding element");
@@ -229,6 +206,13 @@ gst_jpegenc_init (GstJpegEnc * jpegenc)
   jpegenc->cinfo.err = jpeg_std_error (&jpegenc->jerr);
   jpeg_create_compress (&jpegenc->cinfo);
 
+  GST_DEBUG ("gst_jpegenc_init: setting line buffers");
+  jpegenc->line[0] = NULL;
+  jpegenc->line[1] = NULL;
+  jpegenc->line[2] = NULL;
+
+  gst_jpegenc_resync (jpegenc);
+
   jpegenc->jdest.init_destination = gst_jpegenc_init_destination;
   jpegenc->jdest.empty_output_buffer = gst_jpegenc_flush_destination;
   jpegenc->jdest.term_destination = gst_jpegenc_term_destination;
@@ -238,17 +222,6 @@ gst_jpegenc_init (GstJpegEnc * jpegenc)
   jpegenc->smoothing = 0;
 }
 
-static void
-gst_jpegenc_finalize (GObject * object)
-{
-
-  GstJpegEnc *filter = GST_JPEGENC (object);
-
-  jpeg_destroy_compress (&filter->cinfo);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
 static GstCaps *
 gst_jpegenc_getcaps (GstPad * pad)
 {
@@ -318,6 +291,7 @@ gst_jpegenc_link (GstPad * pad, const GstCaps * caps)
 static void
 gst_jpegenc_resync (GstJpegEnc * jpegenc)
 {
+  guint size = 0;
   gint width, height;
 
   GST_DEBUG ("gst_jpegenc_resync: resync");
@@ -337,14 +311,14 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
 #if 0
   switch (jpegenc->format) {
     case GST_COLORSPACE_RGB24:
-      jpegenc->bufsize = jpegenc->width * jpegenc->height * 3;
+      size = 3;
       GST_DEBUG ("gst_jpegenc_resync: setting format to RGB24");
       jpegenc->cinfo.in_color_space = JCS_RGB;
       jpegenc->cinfo.raw_data_in = FALSE;
       break;
     case GST_COLORSPACE_YUV420P:
 #endif
-      jpegenc->bufsize = I420_SIZE (jpegenc->width, jpegenc->height);
+      size = 2;
       jpegenc->cinfo.raw_data_in = TRUE;
       jpegenc->cinfo.in_color_space = JCS_YCbCr;
       GST_DEBUG ("gst_jpegenc_resync: setting format to YUV420P");
@@ -354,16 +328,27 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
       jpegenc->cinfo.comp_info[1].v_samp_factor = 1;
       jpegenc->cinfo.comp_info[2].h_samp_factor = 1;
       jpegenc->cinfo.comp_info[2].v_samp_factor = 1;
+
+      if (height != -1) {
+        jpegenc->line[0] =
+            g_realloc (jpegenc->line[0], height * sizeof (char *));
+        jpegenc->line[1] =
+            g_realloc (jpegenc->line[1], height * sizeof (char *) / 2);
+        jpegenc->line[2] =
+            g_realloc (jpegenc->line[2], height * sizeof (char *) / 2);
+      }
+
       GST_DEBUG ("gst_jpegenc_resync: setting format done");
 #if 0
       break;
     default:
       printf ("gst_jpegenc_resync: unsupported colorspace, using RGB\n");
-      jpegenc->bufsize = jpegenc->width * jpegenc->height * 3;
+      size = 3;
       jpegenc->cinfo.in_color_space = JCS_RGB;
       break;
   }
 #endif
+  jpegenc->bufsize = jpegenc->width * jpegenc->height * size;
 
   jpeg_suppress_tables (&jpegenc->cinfo, TRUE);
   //jpeg_suppress_tables(&jpegenc->cinfo, FALSE);
@@ -380,14 +365,18 @@ gst_jpegenc_chain (GstPad * pad, GstData * _data)
   guchar *data, *outdata;
   gulong size, outsize;
   GstBuffer *outbuf;
-  guint height, width;
-  guchar *base[3], *last[3];
+
+/*  GstMeta *meta; */
+  guint height, width, width2;
+  guchar *base[3];
   gint i, j, k;
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (buf != NULL);
+  /*g_return_if_fail(GST_IS_BUFFER(buf)); */
 
+  /*usleep(10000); */
   jpegenc = GST_JPEGENC (GST_OBJECT_PARENT (pad));
 
   data = GST_BUFFER_DATA (buf);
@@ -405,17 +394,9 @@ gst_jpegenc_chain (GstPad * pad, GstData * _data)
   width = jpegenc->width;
   height = jpegenc->height;
 
-  base[0] = data + I420_Y_OFFSET (width, height);
-  base[1] = data + I420_U_OFFSET (width, height);
-  base[2] = data + I420_V_OFFSET (width, height);
-
-  /* make sure we don't make jpeglib read beyond our buffer,
-   * which might happen if (height % (2*DCTSIZE)) != 0 */
-  last[0] = base[0] + (I420_Y_ROWSTRIDE (width) * ((height / 1) - 1));
-  last[1] =
-      base[1] + (I420_U_ROWSTRIDE (width) * ((ROUND_UP_2 (height) / 2) - 1));
-  last[2] =
-      base[2] + (I420_V_ROWSTRIDE (width) * ((ROUND_UP_2 (height) / 2) - 1));
+  base[0] = data;
+  base[1] = base[0] + width * height;
+  base[2] = base[1] + width * height / 4;
 
   jpegenc->jdest.next_output_byte = outdata;
   jpegenc->jdest.free_in_buffer = outsize;
@@ -424,39 +405,22 @@ gst_jpegenc_chain (GstPad * pad, GstData * _data)
   jpeg_set_quality (&jpegenc->cinfo, jpegenc->quality, TRUE);
   jpeg_start_compress (&jpegenc->cinfo, TRUE);
 
+  width2 = width >> 1;
   GST_DEBUG ("gst_jpegdec_chain: compressing");
 
   for (i = 0; i < height; i += 2 * DCTSIZE) {
-    guchar **lines[3];
-
-    lines[0] = g_malloc ((2 * DCTSIZE) * sizeof (guchar *));
-    lines[1] = g_malloc ((2 * DCTSIZE) * sizeof (guchar *));
-    lines[2] = g_malloc ((2 * DCTSIZE) * sizeof (guchar *));
-    memset (lines[0], 0, (2 * DCTSIZE) * sizeof (guchar *));
-    memset (lines[1], 0, (2 * DCTSIZE) * sizeof (guchar *));
-    memset (lines[2], 0, (2 * DCTSIZE) * sizeof (guchar *));
-
-    /*g_print ("next scanline: %d\n", jpegenc->cinfo.next_scanline); */
-    for (j = 0, k = 0; j < (2 * DCTSIZE); j += 2, k++) {
-      lines[0][j] = base[0];
-      if (base[0] < last[0])
-        base[0] += I420_Y_ROWSTRIDE (width);
-      lines[0][j + 1] = base[0];
-      if (base[0] < last[0])
-        base[0] += I420_Y_ROWSTRIDE (width);
-      lines[1][k] = base[1];
-      if (base[1] < last[1])
-        base[1] += I420_U_ROWSTRIDE (width);
-      lines[2][k] = base[2];
-      if (base[2] < last[2])
-        base[2] += I420_V_ROWSTRIDE (width);
+    for (j = 0, k = 0; j < 2 * DCTSIZE; j += 2, k++) {
+      jpegenc->line[0][j] = base[0];
+      base[0] += width;
+      jpegenc->line[0][j + 1] = base[0];
+      base[0] += width;
+      jpegenc->line[1][k] = base[1];
+      base[1] += width2;
+      jpegenc->line[2][k] = base[2];
+      base[2] += width2;
     }
-    jpeg_write_raw_data (&jpegenc->cinfo, lines, 2 * DCTSIZE);
-    g_free (lines[0]);
-    g_free (lines[1]);
-    g_free (lines[2]);
+    jpeg_write_raw_data (&jpegenc->cinfo, jpegenc->line, 2 * DCTSIZE);
   }
-
   jpeg_finish_compress (&jpegenc->cinfo);
   GST_DEBUG ("gst_jpegdec_chain: compressing done");
 
@@ -512,25 +476,3 @@ gst_jpegenc_get_property (GObject * object, guint prop_id, GValue * value,
       break;
   }
 }
-
-static GstElementStateReturn
-gst_jpegenc_change_state (GstElement * element)
-{
-
-  GstJpegEnc *filter = GST_JPEGENC (element);
-
-  switch (GST_STATE_TRANSITION (element)) {
-    case GST_STATE_NULL_TO_READY:
-      gst_jpegenc_resync (filter);
-      break;
-    case GST_STATE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-
-  if (GST_ELEMENT_CLASS (parent_class)->change_state)
-    return GST_ELEMENT_CLASS (parent_class)->change_state (element);
-
-  return GST_STATE_SUCCESS;
-}
