@@ -58,6 +58,10 @@ typedef struct
   guint state;                  /* state of the pad */
 
   GList *headers;
+
+  gboolean new_page;            /* starting a new page */
+  gboolean first_delta;         /* was the first packet in the page a delta */
+  gboolean prev_delta;          /* was the previous buffer a delta frame */
 }
 GstOggPad;
 
@@ -93,9 +97,10 @@ struct _GstOggMux
   guint64 max_delay;
   guint64 max_page_delay;
 
-  gboolean keyframe_mode;       /* when a delta frame is detected on a stream, we mark
+  GstOggPad *delta_pad;         /* when a delta frame is detected on a stream, we mark
                                    pages as delta frames up to the page that has the
                                    keyframe */
+  guint16 newmediacount;				   
 };
 
 typedef enum
@@ -234,6 +239,7 @@ gst_ogg_mux_get_sink_event_masks (GstPad * pad)
 {
   static const GstEventMask gst_ogg_mux_sink_event_masks[] = {
     {GST_EVENT_EOS, 0},
+    {GST_EVENT_DISCONTINUOUS, 0},
     {0,}
   };
 
@@ -263,6 +269,8 @@ gst_ogg_mux_init (GstOggMux * ogg_mux)
   ogg_mux->max_delay = DEFAULT_MAX_DELAY;
   ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
 
+  ogg_mux->delta_pad = NULL;
+  ogg_mux->newmediacount = 0;
   gst_element_set_loop_function (GST_ELEMENT (ogg_mux), gst_ogg_mux_loop);
 }
 
@@ -288,18 +296,16 @@ static void
 gst_ogg_mux_pad_link (GstPad * pad, GstPad * peer, gpointer data)
 {
   GstOggMux *ogg_mux = ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' connected", padname);
+  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' connected", gst_pad_get_name (pad));
 }
 
 static void
 gst_ogg_mux_pad_unlink (GstPad * pad, GstPad * peer, gpointer data)
 {
   GstOggMux *ogg_mux = ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' unlinked", padname);
+  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' unlinked", gst_pad_get_name (pad));
 }
 
 static GstPad *
@@ -350,6 +356,9 @@ gst_ogg_mux_request_new_pad (GstElement * element,
       oggpad->eos = FALSE;
       /* we assume there will be some control data first for this pad */
       oggpad->state = GST_OGG_PAD_STATE_CONTROL;
+      oggpad->new_page = TRUE;
+      oggpad->first_delta = FALSE;
+      oggpad->prev_delta = FALSE;
 
       /* save a pointer to our data in the pad */
       gst_pad_set_element_private (newpad, oggpad);
@@ -422,6 +431,18 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
         {
           gint64 value = 0;
 
+          if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+            /* only handle if its not first new media event */
+	    if (ogg_mux->newmediacount++ > 0) {
+	      ogg_mux->next_ts = 0;
+              ogg_mux->offset = 0;
+              ogg_mux->pulling = NULL;
+	      pad->offset = 0;
+              GST_DEBUG_OBJECT (ogg_mux, "received new media event");
+	      gst_pad_event_default (pad->pad, event);
+	      break;
+	    }
+          }
           if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
             GST_DEBUG_OBJECT (ogg_mux,
                 "got discont of %" G_GUINT64_FORMAT " on pad %s:%s",
@@ -448,6 +469,7 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
           GST_DEBUG ("muxer: got incaps buffer in control state, ignoring");
           /* just ignore */
           gst_buffer_unref (buf);
+          /* pull next one in next iteration */
           data = NULL;
         } else {
           GST_DEBUG
@@ -462,7 +484,7 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
 }
 
 static GstBuffer *
-gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page)
+gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
 {
   GstBuffer *buffer;
 
@@ -478,15 +500,17 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page)
   GST_BUFFER_OFFSET (buffer) = mux->offset;
   mux->offset += GST_BUFFER_SIZE (buffer);
   GST_BUFFER_OFFSET_END (buffer) = mux->offset;
+  if (delta)
+    GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_DELTA_UNIT);
 
   return buffer;
 }
 
 static void
-gst_ogg_mux_push_page (GstOggMux * mux, ogg_page * page)
+gst_ogg_mux_push_page (GstOggMux * mux, ogg_page * page, gboolean delta)
 {
   if (GST_PAD_IS_USABLE (mux->srcpad)) {
-    GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page);
+    GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
 
     gst_pad_push (mux->srcpad, GST_DATA (buffer));
   }
@@ -704,6 +728,8 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
+    if (packet.granulepos == -1)
+      packet.granulepos = 0;
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
@@ -716,7 +742,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
     GST_LOG ("flushing page with first packet");
     while (ogg_stream_flush (&pad->stream, &page)) {
-      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
+      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
       GST_LOG ("swapped out page");
       hbufs = g_list_append (hbufs, hbuf);
@@ -745,6 +771,8 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
       packet.packet = GST_BUFFER_DATA (buf);
       packet.bytes = GST_BUFFER_SIZE (buf);
       packet.granulepos = GST_BUFFER_OFFSET_END (buf);
+      if (packet.granulepos == -1)
+        packet.granulepos = 0;
       /* mark BOS and packet number */
       packet.b_o_s = (pad->packetno == 0);
       packet.packetno = pad->packetno++;
@@ -760,7 +788,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
         GST_LOG ("flushing page as packet %d is first or last packet",
             pad->packetno);
         while (ogg_stream_flush (&pad->stream, &page)) {
-          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
+          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
           GST_LOG ("swapped out page");
           hbufs = g_list_append (hbufs, hbuf);
@@ -769,7 +797,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
         GST_LOG ("try to swap out page");
         /* just try to swap out a page then */
         while (ogg_stream_pageout (&pad->stream, &page) > 0) {
-          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
+          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
           GST_LOG ("swapped out page");
           hbufs = g_list_append (hbufs, hbuf);
@@ -822,6 +850,7 @@ gst_ogg_mux_loop (GstElement * element)
 {
   GstOggMux *ogg_mux;
   GstOggPad *best;
+  gboolean delta_unit;
 
   ogg_mux = GST_OGG_MUX (element);
 
@@ -840,9 +869,10 @@ gst_ogg_mux_loop (GstElement * element)
       ogg_page page;
 
       while (ogg_stream_flush (&ogg_mux->pulling->stream, &page)) {
-        gst_ogg_mux_push_page (ogg_mux, &page);
+        gst_ogg_mux_push_page (ogg_mux, &page, ogg_mux->pulling->first_delta);
         /* increment the page number counter */
         ogg_mux->pulling->pageno++;
+        ogg_mux->pulling->first_delta = TRUE;
       }
       ogg_mux->pulling = NULL;
     }
@@ -876,6 +906,7 @@ gst_ogg_mux_loop (GstElement * element)
     GstBuffer *buf, *tmpbuf;
     GstOggPad *pad = ogg_mux->pulling;
     gint64 duration;
+    gboolean force_flush;
 
     /* now see if we have a buffer */
     buf = pad->buffer;
@@ -890,12 +921,15 @@ gst_ogg_mux_loop (GstElement * element)
       }
     }
 
+    delta_unit = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_DELTA_UNIT);
     duration = GST_BUFFER_DURATION (buf);
 
     /* create a packet from the buffer */
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
+    if (packet.granulepos == -1)
+      packet.granulepos = 0;
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
@@ -910,6 +944,52 @@ gst_ogg_mux_loop (GstElement * element)
     /* mark EOS */
     packet.e_o_s = (tmpbuf == NULL ? 1 : 0);
 
+    /* we flush when we see a new keyframe */
+    force_flush = (pad->prev_delta && !delta_unit);
+    if (duration != -1) {
+      pad->duration += duration;
+      /* if page duration exceeds max, flush page */
+      if (pad->duration > ogg_mux->max_page_delay) {
+        force_flush = TRUE;
+        pad->duration = 0;
+      }
+    }
+
+    /* force flush */
+    if (force_flush) {
+      while (ogg_stream_flush (&pad->stream, &page)) {
+        gst_ogg_mux_push_page (ogg_mux, &page, pad->first_delta);
+        /* increment the page number counter */
+        pad->pageno++;
+        pad->first_delta = TRUE;
+      }
+      pad->new_page = TRUE;
+    }
+
+    /* if this is the first packet of a new page figure out the delta flag */
+    if (pad->new_page) {
+      if (delta_unit) {
+        /* This page is a delta frame */
+        if (ogg_mux->delta_pad == NULL) {
+          /* we got a delta unit on this pad */
+          ogg_mux->delta_pad = pad;
+        }
+        /* mark the page as delta */
+        pad->first_delta = TRUE;
+      } else {
+        /* got a keyframe */
+        if (ogg_mux->delta_pad == pad) {
+          /* if we get it on the pad with deltaunits,
+           * we mark the page as non delta */
+          pad->first_delta = FALSE;
+        }
+      }
+      pad->new_page = FALSE;
+    }
+
+    /* save key unit to track delta->key unit transitions */
+    pad->prev_delta = delta_unit;
+
     /* swap the packet in */
     ogg_stream_packetin (&pad->stream, &packet);
 
@@ -918,32 +998,20 @@ gst_ogg_mux_loop (GstElement * element)
     /* store new readahead buffer */
     pad->buffer = tmpbuf;
 
-    if (duration != -1) {
-      pad->duration += duration;
-      /* if page duration exceeds max, flush page */
-      if (pad->duration > ogg_mux->max_page_delay) {
-        while (ogg_stream_flush (&pad->stream, &page)) {
-          gst_ogg_mux_push_page (ogg_mux, &page);
-          /* increment the page number counter */
-          pad->pageno++;
-        }
-        pad->duration = 0;
-        return;
-      }
-    }
-
-    /* flush out the pages now. The packet we got could end up in
-     * more than one page so we need to flush them all */
+    /* let ogg write out the pages now. The packet we got could end 
+     * up in more than one page so we need to write them all */
     while (ogg_stream_pageout (&pad->stream, &page) > 0) {
       /* we have a complete page now, we can push the page 
        * and make sure to pull on a new pad the next time around */
-      gst_ogg_mux_push_page (ogg_mux, &page);
+      gst_ogg_mux_push_page (ogg_mux, &page, pad->first_delta);
       /* increment the page number counter */
       pad->pageno++;
       /* we're done pulling on this pad, make sure to choose a new 
        * pad for pulling in the next iteration */
       ogg_mux->pulling = NULL;
+      pad->first_delta = TRUE;
       pad->duration = 0;
+      pad->new_page = TRUE;
     }
   }
 }
