@@ -64,9 +64,7 @@ mulawdec_getcaps (GstPad * pad)
     otherpad = mulawdec->sinkpad;
     base_caps = gst_caps_new_simple ("audio/x-raw-int",
         "width", G_TYPE_INT, 16, "depth", G_TYPE_INT, 16,
-        "signed", G_TYPE_BOOLEAN, TRUE,
-        "endianness", G_TYPE_INT, G_BYTE_ORDER,
-        "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+        "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
   }
   othercaps = gst_pad_get_allowed_caps (otherpad);
 
@@ -95,8 +93,6 @@ mulawdec_link (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   const GValue *rate, *chans;
   GstCaps *base_caps;
-  GstPadLinkReturn link_return;
-
 
   structure = gst_caps_get_structure (caps, 0);
   rate = gst_structure_get_value (structure, "rate");
@@ -111,20 +107,14 @@ mulawdec_link (GstPad * pad, const GstCaps * caps)
     otherpad = mulawdec->srcpad;
     base_caps = gst_caps_new_simple ("audio/x-raw-int",
         "width", G_TYPE_INT, 16, "depth", G_TYPE_INT, 16,
-        "endianness", G_TYPE_INT, G_BYTE_ORDER,
-        "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+        "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
   }
 
   structure = gst_caps_get_structure (base_caps, 0);
   gst_structure_set_value (structure, "rate", rate);
   gst_structure_set_value (structure, "channels", chans);
 
-  link_return = gst_pad_try_set_caps (otherpad, base_caps);
-
-  gst_caps_free (base_caps);
-
-  return link_return;
-
+  return gst_pad_try_set_caps (otherpad, base_caps);
 }
 
 GType
@@ -160,7 +150,7 @@ gst_mulawdec_base_init (GstMuLawDecClass * klass)
     "Mu Law to PCM conversion",
     "Codec/Decoder/Audio",
     "Convert 8bit mu law to 16bit PCM",
-    "Zaheer Abbas Merali <zaheerabbas at merali dot org>"
+    "Zaheer Merali <zaheer@bellworldwide.net>"
   };
 
   gst_element_class_add_pad_template (element_class, mulawdec_src_template);
