@@ -70,7 +70,7 @@ static GstStaticPadTemplate videosink_templ =
         COMMON_VIDEO_CAPS "; "
         "video/x-msmpeg, "
         COMMON_VIDEO_CAPS "; "
-        "image/jpeg, "
+        "video/x-jpeg, "
         COMMON_VIDEO_CAPS "; "
         "video/x-raw-yuv, "
         "format = (fourcc) { YUY2, I420 }, " COMMON_VIDEO_CAPS)
@@ -82,6 +82,7 @@ static GstStaticPadTemplate videosink_templ =
 
 /* FIXME:
  * * audio/x-raw-float: endianness needs defining.
+ * * audio/x-vorbis: private data setup needs work.
  */
 static GstStaticPadTemplate audiosink_templ =
     GST_STATIC_PAD_TEMPLATE ("audio_%d",
@@ -96,15 +97,13 @@ static GstStaticPadTemplate audiosink_templ =
         COMMON_AUDIO_CAPS "; "
         "audio/x-ac3, "
         COMMON_AUDIO_CAPS "; "
-        "audio/x-vorbis, "
-        COMMON_AUDIO_CAPS "; "
         "audio/x-raw-int, "
         "width = (int) { 8, 16, 24 }, "
         "depth = (int) { 8, 16, 24 }, "
         "endianness = (int) { BIG_ENDIAN, LITTLE_ENDIAN }, "
         "signed = (boolean) { true, false }, "
         COMMON_AUDIO_CAPS ";"
-        "audio/x-tta, "
+        "audio/x-raw-tta, "
         "width = (int) { 8, 16, 24 }, "
         "channels = (int) { 1, 2 }, " "rate = (int) [ 8000, 96000 ]")
     );
@@ -115,8 +114,6 @@ GST_STATIC_PAD_TEMPLATE ("subtitle_%d",
     GST_PAD_REQUEST,
     GST_STATIC_CAPS_ANY);
 
-static GArray *used_uids;
-
 /* gobject magic foo */
 static void gst_matroska_mux_base_init (GstMatroskaMuxClass * klass);
 static void gst_matroska_mux_class_init (GstMatroskaMuxClass * klass);
@@ -142,9 +139,6 @@ static void gst_matroska_mux_get_property (GObject * object,
 /* reset muxer */
 static void gst_matroska_mux_reset (GstElement * element);
 
-/* uid generation */
-static guint32 gst_matroska_mux_create_uid ();
-
 static GstEbmlWriteClass *parent_class = NULL;
 
 /*static guint gst_matroska_mux_signals[LAST_SIGNAL] = { 0 };*/
@@ -169,7 +163,7 @@ gst_matroska_mux_get_type (void)
 
     gst_matroska_mux_type =
         g_type_register_static (GST_TYPE_EBML_WRITE,
-        "GstMatroskaMux", &gst_matroska_mux_info, 0);
+        "GstMatroskaMmux", &gst_matroska_mux_info, 0);
   }
 
   return gst_matroska_mux_type;
@@ -248,28 +242,6 @@ gst_matroska_mux_init (GstMatroskaMux * mux)
   gst_matroska_mux_reset (GST_ELEMENT (mux));
 }
 
-static guint32
-gst_matroska_mux_create_uid (void)
-{
-  guint32 uid = 0;
-  GRand *rand = g_rand_new ();
-
-  while (!uid) {
-    guint i;
-
-    uid = g_rand_int (rand);
-    for (i = 0; i < used_uids->len; i++) {
-      if (g_array_index (used_uids, guint32, i) == uid) {
-        uid = 0;
-        break;
-      }
-    }
-    g_array_append_val (used_uids, uid);
-  }
-  g_free (rand);
-  return uid;
-}
-
 static void
 gst_matroska_mux_reset (GstElement * element)
 {
@@ -304,8 +276,6 @@ gst_matroska_mux_reset (GstElement * element)
   mux->num_t_streams = 0;
   mux->num_v_streams = 0;
 
-/* FIXME x-gst-metadata : prehistoric way to do metadata in caps. is done via signals actually */
-
   /* reset media info  (to default) */
   gst_caps_replace (&mux->metadata,
       gst_caps_new_simple ("application/x-gst-metadata",
@@ -319,23 +289,6 @@ gst_matroska_mux_reset (GstElement * element)
   /* reset timers */
   mux->time_scale = 1000000;
   mux->duration = 0;
-
-  /* reset uid array */
-  if (used_uids) {
-    g_array_free (used_uids, TRUE);
-  }
-  /* arbitrary size, 10 should be enough in most cases */
-  used_uids = g_array_sized_new (FALSE, FALSE, sizeof (guint32), 10);
-
-  /* reset cluster */
-  mux->cluster = 0;
-  mux->cluster_time = 0;
-  mux->cluster_pos = 0;
-
-  /* reset meta-seek index */
-  mux->num_meta_indexes = 0;
-  g_free (mux->meta_index);
-  mux->meta_index = NULL;
 }
 
 static GstPadLinkReturn
@@ -404,7 +357,7 @@ gst_matroska_mux_video_pad_link (GstPad * pad, const GstCaps * caps)
     gst_structure_get_fourcc (structure, "format", &videocontext->fourcc);
 
     return GST_PAD_LINK_OK;
-  } else if (!strcmp (mimetype, "image/jpeg")) {
+  } else if (!strcmp (mimetype, "video/x-jpeg")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MJPEG);
 
     return GST_PAD_LINK_OK;
@@ -468,11 +421,9 @@ gst_matroska_mux_video_pad_link (GstPad * pad, const GstCaps * caps)
       case 2:
         context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MPEG2);
         break;
-      case 4:
+      case 3:
         context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_ASP);
         break;
-      default:
-        return FALSE;
     }
 
     return GST_PAD_LINK_OK;
@@ -517,7 +468,6 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
   audiocontext->samplerate = samplerate;
   audiocontext->channels = channels;
   audiocontext->bitdepth = 0;
-  context->default_duration = 0;
 
   if (!strcmp (mimetype, "audio/mpeg")) {
     gint mpegversion = 0;
@@ -531,21 +481,13 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
         switch (layer) {
           case 1:
             context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L1);
-            context->default_duration =
-                384 * GST_SECOND / audiocontext->samplerate;
             break;
           case 2:
             context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L2);
-            context->default_duration =
-                1152 * GST_SECOND / audiocontext->samplerate;
             break;
           case 3:
             context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L3);
-            context->default_duration =
-                1152 * GST_SECOND / audiocontext->samplerate;
             break;
-          default:
-            return FALSE;
         }
         break;
       }
@@ -555,8 +497,6 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
       case 4:
         context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_MPEG4 "MAIN");
         break;
-      default:
-        return FALSE;
     }
 
     return GST_PAD_LINK_OK;
@@ -582,87 +522,14 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
   } else if (!strcmp (mimetype, "audio/x-raw-float")) {
     /* FIXME: endianness is undefined */
   } else if (!strcmp (mimetype, "audio/x-vorbis")) {
-    const GValue *streamheader;
-    guint8 *priv_data = NULL;
-    guint priv_data_size = 0;
-
-    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_VORBIS);
-
-    if (context->codec_priv != NULL) {
-      g_free (context->codec_priv);
-      context->codec_priv = NULL;
-      context->codec_priv_size = 0;
-    }
-    streamheader = gst_structure_get_value (structure, "streamheader");
-
-    if (streamheader != NULL) {
-      if (G_VALUE_TYPE (streamheader) == GST_TYPE_FIXED_LIST) {
-        GArray *bufarr = g_value_peek_pointer (streamheader);
-        gint i;
-        gint offset;
-
-        if (bufarr->len == 3) {
-          GstBuffer *buf[3];
-
-          for (i = 0; i < bufarr->len; i++) {
-            GValue *bufval = &g_array_index (bufarr, GValue, i);
-
-            if (G_VALUE_TYPE (bufval) == GST_TYPE_BUFFER) {
-              buf[i] = g_value_peek_pointer (bufval);
-            }
-          }
-
-          priv_data_size = 1;
-          priv_data_size += GST_BUFFER_SIZE (buf[0]) / 0xff + 1;
-          priv_data_size += GST_BUFFER_SIZE (buf[1]) / 0xff + 1;
-
-          for (i = 0; i < 3; ++i) {
-            priv_data_size += GST_BUFFER_SIZE (buf[i]);
-          }
-
-          priv_data = g_malloc0 (priv_data_size);
-
-          priv_data[0] = 2;
-          offset = 1;
-
-          for (i = 0; i < GST_BUFFER_SIZE (buf[0]) / 0xff; ++i) {
-            priv_data[offset++] = 0xff;
-          }
-          priv_data[offset++] = GST_BUFFER_SIZE (buf[0]) % 0xff;
-
-          for (i = 0; i < GST_BUFFER_SIZE (buf[1]) / 0xff; ++i) {
-            priv_data[offset++] = 0xff;
-          }
-          priv_data[offset++] = GST_BUFFER_SIZE (buf[1]) % 0xff;
-
-          for (i = 0; i < 3; ++i) {
-            memcpy (priv_data + offset, GST_BUFFER_DATA (buf[i]),
-                GST_BUFFER_SIZE (buf[i]));
-            offset += GST_BUFFER_SIZE (buf[i]);
-          }
-        }
-      }
-    }
-
-    if (priv_data == NULL) {
-      /* FIXME:  */
-    }
-
-    context->codec_priv_size = priv_data_size;
-    context->codec_priv = (gpointer) priv_data;
-
-    return GST_PAD_LINK_OK;
-
+    /* FIXME: private data setup needs work */
   } else if (!strcmp (mimetype, "audio/x-ac3")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AC3);
 
     return GST_PAD_LINK_OK;
-  } else if (!strcmp (mimetype, "audio/x-tta")) {
+  } else if (!strcmp (mimetype, "audio/x-raw-tta")) {
     gint width;
 
-    /* TTA frame duration */
-    context->default_duration = 1.04489795918367346939 * GST_SECOND;
-
     gst_structure_get_int (structure, "width", &width);
     audiocontext->bitdepth = width;
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_TTA);
@@ -722,8 +589,8 @@ gst_matroska_mux_request_new_pad (GstElement * element,
 
   pad = gst_pad_new_from_template (templ, name);
   g_free (name);
-  gst_pad_set_link_function (pad, linkfunc);
   gst_element_add_pad (element, pad);
+  gst_pad_set_link_function (pad, linkfunc);
   context->index = mux->num_streams++;
   mux->sink[context->index].track = context;
   context->pad = pad;
@@ -743,19 +610,16 @@ gst_matroska_mux_track_header (GstMatroskaMux * mux,
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKNUMBER, context->num);
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKTYPE, context->type);
 
-  gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKUID,
-      gst_matroska_mux_create_uid ());
-  if (context->default_duration) {
-    gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKDEFAULTDURATION,
-        context->default_duration);
-  }
-
   /* type-specific stuff */
   switch (context->type) {
     case GST_MATROSKA_TRACK_TYPE_VIDEO:{
       GstMatroskaTrackVideoContext *videocontext =
           (GstMatroskaTrackVideoContext *) context;
 
+      /* framerate, but not in the video part */
+      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKDEFAULTDURATION,
+          context->default_duration);
+
       master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_TRACKVIDEO);
       gst_ebml_write_uint (ebml, GST_MATROSKA_ID_VIDEOPIXELWIDTH,
           videocontext->pixel_width);
@@ -824,7 +688,6 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   guint32 seekhead_id[] = { GST_MATROSKA_ID_INFO,
     GST_MATROSKA_ID_TRACKS,
     GST_MATROSKA_ID_CUES,
-    GST_MATROSKA_ID_SEEKHEAD,
 #if 0
     GST_MATROSKA_ID_TAGS,
 #endif
@@ -834,9 +697,6 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   gint i;
   guint tracknum = 1;
   gdouble duration = 0;
-  guint32 *segment_uid = (guint32 *) g_malloc (16);
-  GRand *rand = g_rand_new ();
-  GTimeVal time = { 0, 0 };
 
   /* we start with a EBML header */
   gst_ebml_write_header (ebml, "matroska", 1);
@@ -863,12 +723,6 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   /* segment info */
   mux->info_pos = ebml->pos;
   master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_INFO);
-  for (i = 0; i < 4; i++) {
-    segment_uid[i] = g_rand_int (rand);
-  }
-  g_free (rand);
-  gst_ebml_write_binary (ebml, GST_MATROSKA_ID_SEGMENTUID,
-      (guint8 *) segment_uid, 16);
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TIMECODESCALE, mux->time_scale);
   mux->duration_pos = ebml->pos;
   /* get duration */
@@ -886,7 +740,7 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
       duration / mux->time_scale);
   gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_MUXINGAPP,
-      "GStreamer plugin version " VERSION);
+      "GStreamer plugin version " GST_PLUGINS_VERSION);
   if (mux->metadata
       && gst_structure_has_field (gst_caps_get_structure (mux->metadata, 0),
           "application")) {
@@ -898,8 +752,8 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
       gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_WRITINGAPP, app);
     }
   }
-  g_get_current_time (&time);
-  gst_ebml_write_date (ebml, GST_MATROSKA_ID_DATEUTC, time.tv_sec);
+  /* FIXME: how do I get this? Automatic? Via tags? */
+  /*gst_ebml_write_date (ebml, GST_MATROSKA_ID_DATEUTC, 0); */
   gst_ebml_write_master_finish (ebml, master);
 
   /* tracks */
@@ -927,11 +781,6 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   guint64 duration = 0;
   gint i;
 
-  /* finish last cluster */
-  if (mux->cluster) {
-    gst_ebml_write_master_finish (ebml, mux->cluster);
-  }
-
   /* cues */
   if (mux->index != NULL) {
     guint n;
@@ -946,7 +795,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
 
       pointentry_master = gst_ebml_write_master_start (ebml,
           GST_MATROSKA_ID_POINTENTRY);
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CUETIME,
+      gst_ebml_write_date (ebml, GST_MATROSKA_ID_CUETIME,
           idx->time / mux->time_scale);
       trackpos_master = gst_ebml_write_master_start (ebml,
           GST_MATROSKA_ID_CUETRACKPOSITION);
@@ -961,28 +810,6 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_flush_cache (ebml);
   }
 
-  if (mux->meta_index != NULL) {
-    guint n;
-    guint64 master, seekentry_master;
-
-    mux->meta_pos = ebml->pos;
-    gst_ebml_write_set_cache (ebml, 12 + 28 * mux->num_meta_indexes);
-    master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_SEEKHEAD);
-
-    for (n = 0; n < mux->num_meta_indexes; n++) {
-      GstMatroskaMetaSeekIndex *idx = &mux->meta_index[n];
-
-      seekentry_master = gst_ebml_write_master_start (ebml,
-          GST_MATROSKA_ID_SEEKENTRY);
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_SEEKID, idx->id);
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_SEEKPOSITION,
-          idx->pos - mux->segment_master);
-      gst_ebml_write_master_finish (ebml, seekentry_master);
-    }
-    gst_ebml_write_master_finish (ebml, master);
-  }
-  gst_ebml_write_flush_cache (ebml);
-
   /* FIXME: tags */
 
   /* update seekhead. We know that:
@@ -1010,17 +837,6 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
     gst_ebml_write_seek (ebml, my_pos);
   }
-  if (mux->meta_index != NULL) {
-    gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
-        mux->meta_pos - mux->segment_master);
-  } else {
-    /* void'ify */
-    guint64 my_pos = ebml->pos;
-
-    gst_ebml_write_seek (ebml, mux->seekhead_pos + 96);
-    gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
-    gst_ebml_write_seek (ebml, my_pos);
-  }
 #if 0
   gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
       mux->tags_pos - mux->segment_master);
@@ -1051,6 +867,7 @@ gst_matroska_mux_prepare_data (GstMatroskaMux * mux)
 
   for (i = 0; i < mux->num_streams; i++) {
     while (!mux->sink[i].eos && !mux->sink[i].buffer &&
+        mux->sink[i].track->num > 0 &&
         GST_PAD_IS_USABLE (mux->sink[i].track->pad)) {
       GstData *data;
 
@@ -1097,56 +914,15 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux)
   buf = mux->sink[i].buffer;
   mux->sink[i].buffer = NULL;
 
-  if (mux->cluster) {
-    /* start a new cluster every two seconds */
-    if (mux->cluster_time + GST_SECOND * 2 < GST_BUFFER_TIMESTAMP (buf)) {
-      GstMatroskaMetaSeekIndex *idx;
-
-      gst_ebml_write_master_finish (ebml, mux->cluster);
-      mux->cluster_pos = ebml->pos;
-      mux->cluster =
-          gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
-          GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
-      mux->cluster_time = GST_BUFFER_TIMESTAMP (buf);
-
-      if (mux->num_meta_indexes % 32 == 0) {
-        mux->meta_index = g_renew (GstMatroskaMetaSeekIndex, mux->meta_index,
-            mux->num_meta_indexes + 32);
-      }
-      idx = &mux->meta_index[mux->num_meta_indexes++];
-      idx->id = GST_MATROSKA_ID_CLUSTER;
-      idx->pos = mux->cluster_pos;
-    }
-  } else {
-    /* first cluster */
-    GstMatroskaMetaSeekIndex *idx;
-
-    mux->cluster_pos = ebml->pos;
-    mux->cluster = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
-    gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
-        GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
-    mux->cluster_time = GST_BUFFER_TIMESTAMP (buf);
-
-    if (mux->num_meta_indexes % 32 == 0) {
-      mux->meta_index = g_renew (GstMatroskaMetaSeekIndex, mux->meta_index,
-          mux->num_meta_indexes + 32);
-    }
-    idx = &mux->meta_index[mux->num_meta_indexes++];
-    idx->id = GST_MATROSKA_ID_CLUSTER;
-    idx->pos = mux->cluster_pos;
-  }
-  cluster = mux->cluster;
-
   /* update duration of this track */
   if (GST_BUFFER_DURATION_IS_VALID (buf))
     mux->sink[i].duration += GST_BUFFER_DURATION (buf);
 
   /* We currently write an index entry for each keyframe in a
-   * video track or one entry for each cluster in an audio track
-   * for audio only files. This can be largely improved, such as doing
+   * video track. This can be largely improved, such as doing
    * one for each keyframe or each second (for all-keyframe
-   * streams), only the *first* video track. But that'll come later... */
+   * streams), only the *first* video track or the audio track
+   * if we have no video tracks. But that'll come later... */
   if (mux->sink[i].track->type == GST_MATROSKA_TRACK_TYPE_VIDEO &&
       GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_KEY_UNIT)) {
     GstMatroskaIndex *idx;
@@ -1157,49 +933,31 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux)
     }
     idx = &mux->index[mux->num_indexes++];
 
-    idx->pos = mux->cluster_pos;
-    idx->time = GST_BUFFER_TIMESTAMP (buf);
-    idx->track = mux->sink[i].track->num;
-  } else if ((mux->sink[i].track->type == GST_MATROSKA_TRACK_TYPE_AUDIO) &&
-      (mux->num_streams == 1)) {
-    GstMatroskaIndex *idx;
-
-    if (mux->num_indexes % 32 == 0) {
-      mux->index = g_renew (GstMatroskaIndex, mux->index,
-          mux->num_indexes + 32);
-    }
-    idx = &mux->index[mux->num_indexes++];
-
-    idx->pos = mux->cluster_pos;
+    idx->pos = ebml->pos;
     idx->time = GST_BUFFER_TIMESTAMP (buf);
     idx->track = mux->sink[i].track->num;
   }
 
-  /* write one blockgroup with one block with
+  /* write one cluster with one blockgroup with one block with
    * one slice (*breath*).
-   * FIXME: lacing, etc. */
+   * FIXME: lacing, multiple frames/cluster, etc. */
+  cluster = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
+  gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
+      GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
   blockgroup = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_BLOCKGROUP);
   gst_ebml_write_buffer_header (ebml, GST_MATROSKA_ID_BLOCK,
       GST_BUFFER_SIZE (buf) + 4);
   hdr = gst_buffer_new_and_alloc (4);
   /* track num - FIXME: what if num >= 0x80 (unlikely)? */
   GST_BUFFER_DATA (hdr)[0] = mux->sink[i].track->num | 0x80;
-  /* time relative to clustertime */
-  *(guint16 *) & GST_BUFFER_DATA (hdr)[1] = GUINT16_TO_BE (
-      (GST_BUFFER_TIMESTAMP (buf) - mux->cluster_time) / mux->time_scale);
+  /* time relative to clustertime - we don't use this yet */
+  *(guint16 *) & GST_BUFFER_DATA (hdr)[1] = GUINT16_TO_BE (0);
   /* flags - no lacing (yet) */
   GST_BUFFER_DATA (hdr)[3] = 0;
   gst_ebml_write_buffer (ebml, hdr);
   gst_ebml_write_buffer (ebml, buf);
-  if (GST_BUFFER_DURATION_IS_VALID (buf)) {
-    guint64 block_duration = GST_BUFFER_DURATION (buf);
-
-    if (block_duration != mux->sink[i].track->default_duration) {
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_BLOCKDURATION,
-          block_duration / mux->time_scale);
-    }
-  }
   gst_ebml_write_master_finish (ebml, blockgroup);
+  gst_ebml_write_master_finish (ebml, cluster);
 }
 
 static void
@@ -1208,33 +966,15 @@ gst_matroska_mux_loop (GstElement * element)
   GstMatroskaMux *mux = GST_MATROSKA_MUX (element);
   guint i;
 
-  if (gst_matroska_mux_prepare_data (mux) == -1) {
-    gboolean eos = TRUE;
-
-    for (i = 0; i < mux->num_streams; i++) {
-      eos = eos & mux->sink[i].eos;
-    }
-    if (eos) {
-      // all streams are EOS, finish writing
-      gst_matroska_mux_write_data (mux);
-    } else {
-      // no data available and no EOS
-      GST_ELEMENT_ERROR (element, STREAM, MUX, (NULL), ("No data"));
-    }
-    return;
-  }
-
   /* start with a header */
   if (mux->state == GST_MATROSKA_MUX_STATE_START) {
     if (mux->num_streams == 0) {
-      GST_ELEMENT_ERROR (mux, STREAM, MUX, (NULL),
-          ("No input streams configured"));
       return;
     }
     for (i = 0; i < mux->num_streams; i++) {
       if (!gst_pad_is_negotiated (mux->sink[i].track->pad)) {
-        GST_ELEMENT_ERROR (mux, CORE, NEGOTIATION, (NULL), (NULL));
         return;
+      } else {
       }
     }
     mux->state = GST_MATROSKA_MUX_STATE_HEADER;
