@@ -91,7 +91,7 @@ enum
 {
   MODPLUG_STATE_NEED_TUNE = 1,
   MODPLUG_STATE_LOAD_TUNE = 2,
-  MODPLUG_STATE_PLAY_TUNE = 3,
+  MODPLUG_STATE_PLAY_TUNE = 3
 };
 
 static void gst_modplug_base_init (GstModPlugClass * klass);
@@ -133,6 +133,7 @@ gst_modplug_get_type (void)
       (GInstanceInitFunc) gst_modplug_init,
       NULL
     };
+
     modplug_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstModPlug", &modplug_info,
         (GTypeFlags) 0);
