@@ -21,7 +21,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <gstmpegdemux.h>
+#include "gstmpegdemux.h"
 
 
 GST_DEBUG_CATEGORY_STATIC (gstmpegdemux_debug);
@@ -51,8 +51,8 @@ enum
 {
   ARG_0,
   ARG_BIT_RATE,
-  ARG_MPEG2,
-  /* FILL ME */
+  ARG_MPEG2
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -331,7 +331,6 @@ gst_mpeg_demux_new_output_pad (GstMPEGDemux * mpeg_demux,
   GstPad *pad;
 
   pad = gst_pad_new_from_template (temp, name);
-  gst_element_add_pad (GST_ELEMENT (mpeg_demux), pad);
 
   gst_pad_set_formats_function (pad, gst_mpeg_demux_get_src_formats);
   gst_pad_set_convert_function (pad, gst_mpeg_parse_convert_src);
@@ -341,6 +340,8 @@ gst_mpeg_demux_new_output_pad (GstMPEGDemux * mpeg_demux,
   gst_pad_set_query_function (pad, gst_mpeg_parse_handle_src_query);
   gst_pad_use_explicit_caps (pad);
 
+  gst_element_add_pad (GST_ELEMENT (mpeg_demux), pad);
+
   return pad;
 }
 
@@ -369,6 +370,7 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
   GstMPEGVideoStream *video_str;
   gchar *name;
   GstCaps *caps;
+  gboolean set_caps = FALSE;
 
   g_return_val_if_fail (stream_nr < GST_MPEG_DEMUX_NUM_VIDEO_STREAMS, NULL);
   g_return_val_if_fail (type > GST_MPEG_DEMUX_VIDEO_UNKNOWN &&
@@ -379,7 +381,6 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
   if (str == NULL) {
     video_str = g_new0 (GstMPEGVideoStream, 1);
     str = (GstMPEGStream *) video_str;
-    str->type = GST_MPEG_DEMUX_VIDEO_UNKNOWN;
 
     name = g_strdup_printf ("video_%02d", stream_nr);
     CLASS (mpeg_demux)->init_stream (mpeg_demux, type, str, stream_nr, name,
@@ -387,6 +388,8 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     g_free (name);
 
     mpeg_demux->video_stream[stream_nr] = str;
+
+    set_caps = TRUE;
   } else {
     /* This stream may have been created by a derived class, reset the
        size. */
@@ -394,16 +397,20 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     mpeg_demux->video_stream[stream_nr] = str = (GstMPEGStream *) video_str;
   }
 
-  if (str->type != GST_MPEG_DEMUX_VIDEO_MPEG ||
-      video_str->mpeg_version != mpeg_version) {
+  if (set_caps || video_str->mpeg_version != mpeg_version) {
     /* We need to set new caps for this pad. */
     caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, mpeg_version,
         "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
-    gst_pad_set_explicit_caps (str->pad, caps);
+    if (!gst_pad_set_explicit_caps (str->pad, caps)) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+          CORE, NEGOTIATION, (NULL), ("failed to set caps"));
+      gst_caps_free (caps);
+      return str;
+    }
+    gst_caps_free (caps);
 
     /* Store the current values. */
-    str->type = GST_MPEG_DEMUX_VIDEO_MPEG;
     video_str->mpeg_version = mpeg_version;
   }
 
@@ -417,6 +424,7 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
   GstMPEGStream *str;
   gchar *name;
   GstCaps *caps;
+  gboolean set_caps = FALSE;
 
   g_return_val_if_fail (stream_nr < GST_MPEG_DEMUX_NUM_AUDIO_STREAMS, NULL);
   g_return_val_if_fail (type > GST_MPEG_DEMUX_AUDIO_UNKNOWN &&
@@ -426,7 +434,6 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
 
   if (str == NULL) {
     str = g_new0 (GstMPEGStream, 1);
-    str->type = GST_MPEG_DEMUX_AUDIO_MPEG;
 
     name = g_strdup_printf ("audio_%02d", stream_nr);
     CLASS (mpeg_demux)->init_stream (mpeg_demux, type, str, stream_nr, name,
@@ -434,19 +441,26 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
     g_free (name);
 
     mpeg_demux->audio_stream[stream_nr] = str;
+
+    /* new pad, set caps */
+    set_caps = TRUE;
   } else {
     /* This stream may have been created by a derived class, reset the
        size. */
     str = g_renew (GstMPEGStream, str, 1);
   }
 
-  if (str->type != GST_MPEG_DEMUX_AUDIO_MPEG) {
+  if (set_caps) {
     /* We need to set new caps for this pad. */
     caps = gst_caps_new_simple ("audio/mpeg",
         "mpegversion", G_TYPE_INT, 1, NULL);
-    gst_pad_set_explicit_caps (str->pad, caps);
-
-    str->type = GST_MPEG_DEMUX_AUDIO_MPEG;
+    if (!gst_pad_set_explicit_caps (str->pad, caps)) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+          CORE, NEGOTIATION, (NULL), ("failed to set caps"));
+      gst_caps_free (caps);
+      return str;
+    }
+    gst_caps_free (caps);
   }
 
   return str;
@@ -499,7 +513,7 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   buf = GST_BUFFER_DATA (buffer);
   buf += 4;
 
-  header_length = GUINT16_FROM_BE (*(guint16 *) buf);
+  header_length = GST_READ_UINT16_BE (buf);
   GST_DEBUG_OBJECT (mpeg_demux, "header_length %d", header_length);
   buf += 2;
 
@@ -570,7 +584,7 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
         outstream = CLASS (mpeg_demux)->get_video_stream (mpeg_demux,
             stream_id - 0xE0, GST_MPEG_DEMUX_VIDEO_MPEG, &mpeg_version);
       } else {
-        GST_WARNING_OBJECT (mpeg_demux, "unkown stream id 0x%02x", stream_id);
+        GST_WARNING_OBJECT (mpeg_demux, "unknown stream id 0x%02x", stream_id);
       }
 
       GST_DEBUG_OBJECT (mpeg_demux, "STD_buffer_bound_scale %d",
@@ -632,7 +646,7 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   buf += 4;
 
   /* start parsing */
-  packet_length = GUINT16_FROM_BE (*((guint16 *) buf));
+  packet_length = GST_READ_UINT16_BE (buf);
 
   GST_DEBUG_OBJECT (mpeg_demux, "got packet_length %d", packet_length);
   headerlen = 2;
@@ -714,6 +728,7 @@ done:
 
   /* calculate the amount of real data in this packet */
   datalen = packet_length - headerlen + 2;
+
   GST_DEBUG_OBJECT (mpeg_demux, "headerlen is %d, datalen is %d",
       headerlen, datalen);
 
@@ -752,7 +767,7 @@ done:
     CLASS (mpeg_demux)->send_subbuffer (mpeg_demux, outstream, buffer,
         timestamp, headerlen + 4, datalen);
   } else {
-    GST_WARNING_OBJECT (mpeg_demux, "unkown stream id 0x%02x", id);
+    GST_WARNING_OBJECT (mpeg_demux, "unknown stream id 0x%02x", id);
   }
 
   return TRUE;
@@ -779,7 +794,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   buf += 4;
 
   /* start parsing */
-  packet_length = GUINT16_FROM_BE (*((guint16 *) buf));
+  packet_length = GST_READ_UINT16_BE (buf);
 
   GST_DEBUG_OBJECT (mpeg_demux, "packet_length %d", packet_length);
   buf += 2;
@@ -881,7 +896,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
     CLASS (mpeg_demux)->send_subbuffer (mpeg_demux, outstream, buffer,
         timestamp, headerlen + 4, datalen);
   } else {
-    GST_WARNING_OBJECT (mpeg_demux, "unkown stream id 0x%02x", id);
+    GST_WARNING_OBJECT (mpeg_demux, "unknown stream id 0x%02x", id);
   }
 
   return TRUE;
