@@ -201,13 +201,6 @@ static GstTagEntryMatch tag_matches[] = {
   {GST_TAG_TRACK_PEAK, "REPLAYGAIN_TRACK_PEAK"},
   {GST_TAG_ALBUM_GAIN, "REPLAYGAIN_ALBUM_GAIN"},
   {GST_TAG_ALBUM_PEAK, "REPLAYGAIN_ALBUM_PEAK"},
-  {GST_TAG_MUSICBRAINZ_TRACKID, "MUSICBRAINZ_TRACKID"},
-  {GST_TAG_MUSICBRAINZ_ARTISTID, "MUSICBRAINZ_ARTISTID"},
-  {GST_TAG_MUSICBRAINZ_ALBUMID, "MUSICBRAINZ_ALBUMID"},
-  {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "MUSICBRAINZ_ALBUMARTISTID"},
-  {GST_TAG_MUSICBRAINZ_TRMID, "MUSICBRAINZ_TRMID"},
-  {GST_TAG_MUSICBRAINZ_SORTNAME, "MUSICBRAINZ_SORTNAME"},
-  {GST_TAG_LANGUAGE_CODE, "LANGUAGE"},
   {NULL, NULL}
 };
 
@@ -327,24 +320,13 @@ gst_vorbis_tag_add (GstTagList * list, const gchar * tag, const gchar * value)
       }
       break;
     case G_TYPE_STRING:{
-      gchar *valid = NULL;
+      gchar *valid;
 
-      /* specialcase for language code */
-      if (strcmp (tag, "LANGUAGE") == 0) {
-        const gchar *s = strchr (value, '[');
-
-        if (s && strchr (s, ']') == s + 4) {
-          valid = g_strndup (s + 1, 3);
-        }
-      }
-
-      if (!valid) {
-        if (!g_utf8_validate (value, -1, (const gchar **) &valid)) {
-          valid = g_strndup (value, valid - value);
-          GST_DEBUG ("Invalid vorbis comment tag, truncated it to %s", valid);
-        } else {
-          valid = g_strdup (value);
-        }
+      if (!g_utf8_validate (value, -1, (const gchar **) &valid)) {
+        valid = g_strndup (value, valid - value);
+        GST_DEBUG ("Invalid vorbis comment tag, truncated it to %s\n", valid);
+      } else {
+        valid = g_strdup (value);
       }
       gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, valid, NULL);
       g_free (valid);
@@ -385,7 +367,7 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer * buffer,
   data += 4;									\
   size -= 4;									\
   if (cur_size > size) goto error;						\
-  cur = (gchar*) data;								\
+  cur = data;									\
 }G_STMT_END
   gchar *cur, *value;
   guint cur_size;
@@ -486,7 +468,6 @@ gst_tag_to_vorbis_comments (const GstTagList * list, const gchar * tag)
         if (!gst_tag_list_get_string_index (list, tag, i, &str))
           g_assert_not_reached ();
         result = g_strdup_printf ("%s=%s", vorbis_tag, str);
-        g_free (str);
         break;
       }
       case G_TYPE_DOUBLE:{
@@ -495,7 +476,6 @@ gst_tag_to_vorbis_comments (const GstTagList * list, const gchar * tag)
         if (!gst_tag_list_get_double_index (list, tag, i, &value))
           g_assert_not_reached ();
         result = g_strdup_printf ("%s=%f", vorbis_tag, value);
-        break;
       }
       default:
         GST_DEBUG ("Couldn't write tag %s", tag);
@@ -584,7 +564,6 @@ gst_tag_list_to_vorbiscomment_buffer (const GstTagList * list,
     memcpy (data, cur, size);
     data += size;
   }
-  g_list_foreach (my_data.entries, (GFunc) g_free, NULL);
   g_list_free (my_data.entries);
   *data = 1;
 
@@ -600,28 +579,22 @@ gst_vorbis_tag_chain (GstPad * pad, GstData * data)
   tag = GST_VORBIS_TAG (gst_pad_get_parent (pad));
 
   if (tag->output == OUTPUT_UNKNOWN) {
-    GstCaps *vorbis_caps = gst_caps_new_simple ("audio/x-vorbis", NULL);
-    GstCaps *tags_caps = gst_caps_new_simple ("application/x-gst-tags", NULL);
-
     /* caps nego */
     do {
-      if (gst_pad_try_set_caps (tag->srcpad, vorbis_caps) >= 0) {
+      if (gst_pad_try_set_caps (tag->srcpad,
+              gst_caps_new_simple ("audio/x-vorbis", NULL)) >= 0) {
         tag->output = OUTPUT_DATA;
-      } else if (gst_pad_try_set_caps (tag->srcpad, tags_caps) >= 0) {
+      } else if (gst_pad_try_set_caps (tag->srcpad,
+              gst_caps_new_simple ("application/x-gst-tags", NULL)) >= 0) {
         tag->output = OUTPUT_TAGS;
       } else {
         const GstCaps *caps =
             gst_static_caps_get (&gst_vorbis_tag_src_template.static_caps);
         if (gst_pad_recover_caps_error (tag->srcpad, caps))
           continue;
-        gst_caps_free (vorbis_caps);
-        gst_caps_free (tags_caps);
         return;
       }
     } while (FALSE);
-
-    gst_caps_free (vorbis_caps);
-    gst_caps_free (tags_caps);
   }
 
   if (GST_BUFFER_SIZE (buffer) == 0)
@@ -630,9 +603,9 @@ gst_vorbis_tag_chain (GstPad * pad, GstData * data)
 
   if (GST_BUFFER_DATA (buffer)[0] == 3) {
     gchar *vendor;
-    GstTagList *list = gst_tag_list_from_vorbiscomment_buffer (buffer,
-        (const guint8 *) "\003vorbis", 7, &vendor);
-    const GstTagList *found_tags;
+    GstTagList *list =
+        gst_tag_list_from_vorbiscomment_buffer (buffer, "\003vorbis", 7,
+        &vendor);
 
     gst_data_unref (data);
     if (list == NULL) {
@@ -642,13 +615,11 @@ gst_vorbis_tag_chain (GstPad * pad, GstData * data)
     }
     gst_element_found_tags_for_pad (GST_ELEMENT (tag), tag->srcpad, 0,
         gst_tag_list_copy (list));
-    found_tags = gst_tag_setter_get_list (GST_TAG_SETTER (tag));
-    if (found_tags)
-      gst_tag_list_insert (list, found_tags,
-          gst_tag_setter_get_merge_mode (GST_TAG_SETTER (tag)));
+    gst_tag_list_merge (list, gst_tag_setter_get_list (GST_TAG_SETTER (tag)),
+        gst_tag_setter_get_merge_mode (GST_TAG_SETTER (tag)));
     data =
-        GST_DATA (gst_tag_list_to_vorbiscomment_buffer (list,
-            (const guint8 *) "\003vorbis", 7, vendor));
+        GST_DATA (gst_tag_list_to_vorbiscomment_buffer (list, "\003vorbis", 7,
+            vendor));
     gst_tag_list_free (list);
     g_free (vendor);
   }
