@@ -63,13 +63,6 @@ typedef enum
   GST_RECOVER_POLICY_RESYNC_KEYFRAME,
 } GstRecoverPolicy;
 
-typedef enum
-{
-  GST_SYNC_METHOD_NONE,
-  GST_SYNC_METHOD_WAIT,
-  GST_SYNC_METHOD_BURST,
-} GstSyncMethod;
-
 typedef enum
 {
   GST_UNIT_TYPE_BUFFERS,
@@ -84,8 +77,6 @@ typedef enum
   GST_CLIENT_STATUS_REMOVED	= 2,
   GST_CLIENT_STATUS_SLOW	= 3,
   GST_CLIENT_STATUS_ERROR	= 4,
-  GST_CLIENT_STATUS_DUPLICATE	= 5,
-  GST_CLIENT_STATUS_NULLSTATE   = 6,
 } GstClientStatus;
 
 /* structure for a client
@@ -107,9 +98,7 @@ typedef struct {
 
   gboolean caps_sent;
   gboolean streamheader_sent;
-  gboolean new_connection;
-
-  gint buffers_remaining; /* remaining buffers to send to this client */
+  gboolean need_keyunit;
 
   /* stats */
   guint64 bytes_sent;
@@ -118,14 +107,9 @@ typedef struct {
   guint64 last_activity_time;
   guint64 dropped_buffers;
   guint64 avg_queue_size;
-
+  
 } GstTCPClient;
 
-#define CLIENTS_LOCK_INIT(fdsink)	(g_static_rec_mutex_init(&fdsink->clientslock))
-#define CLIENTS_LOCK_FREE(fdsink)	(g_static_rec_mutex_free(&fdsink->clientslock))
-#define CLIENTS_LOCK(fdsink)		(g_static_rec_mutex_lock(&fdsink->clientslock))
-#define CLIENTS_UNLOCK(fdsink)		(g_static_rec_mutex_unlock(&fdsink->clientslock))
-
 struct _GstMultiFdSink {
   GstElement element;
 
@@ -135,18 +119,18 @@ struct _GstMultiFdSink {
   guint64 bytes_to_serve; /* how much bytes we must serve */
   guint64 bytes_served; /* how much bytes have we served */
 
-  GStaticRecMutex clientslock;	/* lock to protect the clients list */
+  GMutex *clientslock;	/* lock to protect the clients list */
   GList *clients;	/* list of clients we are serving */
-  GHashTable *fd_hash;  /* index on fd to client */
-
+  
   GstFDSetMode mode;
   GstFDSet *fdset;
 
+  //fd_set readfds; /* all the client file descriptors that we can read from */
+  //fd_set writefds; /* all the client file descriptors that we can write to */
+
   GstFD control_sock[2];/* sockets for controlling the select call */
 
   GSList *streamheader; /* GSList of GstBuffers to use as streamheader */
-  gboolean previous_buffer_in_caps;
-
   GstTCPProtocolType protocol;
   guint mtu;
 
@@ -160,16 +144,12 @@ struct _GstMultiFdSink {
   gint units_soft_max;	/* max units a client can lag before recovery starts */
   GstRecoverPolicy recover_policy;
   GstClockTime timeout;	/* max amount of nanoseconds to remain idle */
-  GstSyncMethod sync_method;	/* what method to use for connecting clients */
-
-  gint buffers_to_send; /* maximum number of buffer to send to a client */
+  gboolean sync_clients;/* sync clients to keyframe */
 
   /* stats */
   gint buffers_queued;	/* number of queued buffers */
   gint bytes_queued;	/* number of queued bytes */
   gint time_queued;	/* number of queued time */
-
-  gboolean buffer_processed_signal;
 };
 
 struct _GstMultiFdSinkClass {
@@ -185,12 +165,10 @@ struct _GstMultiFdSinkClass {
   gboolean (*init)   (GstMultiFdSink *sink);
   gboolean (*wait)   (GstMultiFdSink *sink, GstFDSet *set);
   gboolean (*close)  (GstMultiFdSink *sink);
-  void (*removed) (GstMultiFdSink *sink, int fd);
 
   /* signals */
   void (*client_added) (GstElement *element, gchar *host, gint fd);
   void (*client_removed) (GstElement *element, gchar *host, gint fd);
-  void (*buffer_processed) (GstElement *element, gint fd);
 };
 
 GType gst_multifdsink_get_type (void);
