@@ -62,6 +62,7 @@ gst_udpsrc_control_get_type (void)
     {CONTROL_TCP, "3", "tcp"},
     {CONTROL_ZERO, NULL, NULL},
   };
+
   if (!udpsrc_control_type) {
     udpsrc_control_type =
         g_enum_register_static ("GstUDPSrcControl", udpsrc_control);
@@ -104,6 +105,7 @@ gst_udpsrc_get_type (void)
       (GInstanceInitFunc) gst_udpsrc_init,
       NULL
     };
+
     udpsrc_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstUDPSrc", &udpsrc_info, 0);
   }
@@ -173,6 +175,7 @@ gst_udpsrc_init (GstUDPSrc * udpsrc)
   udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
 
   udpsrc->first_buf = TRUE;
+  udpsrc->defer_data = NULL;
 }
 
 static GstData *
@@ -191,6 +194,13 @@ gst_udpsrc_get (GstPad * pad)
 
   udpsrc = GST_UDPSRC (GST_OBJECT_PARENT (pad));
 
+  if (udpsrc->defer_data != NULL) {
+    GstData *outdata = udpsrc->defer_data;
+
+    udpsrc->defer_data = NULL;
+    return outdata;
+  }
+
   FD_ZERO (&read_fds);
   FD_SET (udpsrc->sock, &read_fds);
   if (udpsrc->control != CONTROL_NONE) {
@@ -252,9 +262,9 @@ gst_udpsrc_get (GstPad * pad)
         GST_ELEMENT_ERROR (udpsrc, CORE, NEGOTIATION, (NULL),
             ("Got unfixed caps from peer"));
       }
-
-#endif
       g_free (buf);
+#endif
+
       outbuf = NULL;
     } else {
       outbuf = gst_buffer_new ();
@@ -273,7 +283,7 @@ gst_udpsrc_get (GstPad * pad)
           discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
               current_time, NULL);
 
-          gst_pad_push (udpsrc->srcpad, GST_DATA (discont));
+          udpsrc->defer_data = GST_DATA (discont);
         }
 
         udpsrc->first_buf = FALSE;
@@ -294,12 +304,21 @@ gst_udpsrc_get (GstPad * pad)
         gst_buffer_unref (outbuf);
         outbuf = NULL;
       }
-
     }
   } else {
     perror ("select");
     outbuf = NULL;
   }
+  if (udpsrc->defer_data) {
+    GstData *databuf = udpsrc->defer_data;
+
+    udpsrc->defer_data = GST_DATA (outbuf);
+    return databuf;
+  }
+
+  if (outbuf == NULL)
+    return GST_DATA (gst_event_new (GST_EVENT_EMPTY));
+
   return GST_DATA (outbuf);
 }
 
